@@ -23,7 +23,7 @@ use rayon::prelude::*;
 use reqwest::{
     self,
     header::{ACCEPT, CONTENT_TYPE},
-    Body, StatusCode as ResponseStatusCode,
+    StatusCode as ResponseStatusCode,
 };
 use serde::{Deserialize, Serialize};
 use serde_json::{json, Value};
@@ -73,7 +73,6 @@ pub fn upload_bundles_stream<'a>(
     price_terms: (u64, u64),
     buffer: usize,
 ) -> impl Stream<Item = Result<BundleStatus, Error>> + 'a {
-    // let path_chunks = arweave.chunk_file_paths(paths_iter, data_size).unwrap();
     stream::iter(paths_chunks)
         .map(move |p| arweave.post_bundle_transaction_from_file_paths(p, tags.clone(), price_terms))
         .buffer_unordered(buffer)
@@ -89,7 +88,6 @@ pub fn upload_bundles_stream_with_sol<'a>(
     sol_ar_url: Url,
     from_keypair: &'a Keypair,
 ) -> impl Stream<Item = Result<BundleStatus, Error>> + 'a {
-    // let path_chunks = arweave.chunk_file_paths(paths_iter, data_size).unwrap();
     stream::iter(paths_chunks)
         .map(move |p| {
             arweave.post_bundle_transaction_from_file_paths_with_sol(
@@ -338,9 +336,10 @@ impl Arweave {
         other_tags: Option<Vec<Tag<Base64>>>,
         last_tx: Option<Base64>,
         price_terms: (u64, u64),
+        auto_content_tag: bool,
     ) -> Result<Transaction, Error> {
         let data = fs::read(file_path).await?;
-        self.create_transaction(data, other_tags, last_tx, price_terms, true)
+        self.create_transaction(data, other_tags, last_tx, price_terms, auto_content_tag)
             .await
     }
 
@@ -562,7 +561,7 @@ impl Arweave {
             .fold(serde_json::Map::new(), |mut m, s| {
                 m.insert(
                     s.file_path.unwrap().display().to_string(),
-                    json!({"id": s.id.to_string()}),
+                    json!({"id": s.id.to_string(), "content_type": s.content_type}),
                 );
                 m
             });
@@ -665,16 +664,33 @@ impl Arweave {
         &self,
         file_path: PathBuf,
         log_dir: Option<PathBuf>,
-        additional_tags: Option<Vec<Tag<Base64>>>,
+        mut additional_tags: Option<Vec<Tag<Base64>>>,
         last_tx: Option<Base64>,
         price_terms: (u64, u64),
     ) -> Result<Status, Error> {
+        let mut auto_content_tag = true;
+        let mut status_content_type = mime_guess::mime::OCTET_STREAM.to_string();
+
+        if let Some(content_type) = mime_guess::from_path(file_path.clone()).first() {
+            status_content_type = content_type.to_string();
+            auto_content_tag = false;
+            let content_tag: Tag<Base64> =
+                Tag::from_utf8_strs("Content-Type", &content_type.to_string())?;
+            if let Some(mut tags) = additional_tags {
+                tags.push(content_tag);
+                additional_tags = Some(tags);
+            } else {
+                additional_tags = Some(vec![content_tag]);
+            }
+        }
+
         let transaction = self
             .create_transaction_from_file_path(
                 file_path.clone(),
                 additional_tags,
                 last_tx,
                 price_terms,
+                auto_content_tag,
             )
             .await?;
         let signed_transaction = self.sign_transaction(transaction)?;
@@ -684,6 +700,7 @@ impl Arweave {
             id,
             reward,
             file_path: Some(file_path),
+            content_type: status_content_type,
             ..Default::default()
         };
 
@@ -717,19 +734,36 @@ impl Arweave {
         &self,
         file_path: PathBuf,
         log_dir: Option<PathBuf>,
-        additional_tags: Option<Vec<Tag<Base64>>>,
+        mut additional_tags: Option<Vec<Tag<Base64>>>,
         last_tx: Option<Base64>,
         price_terms: (u64, u64),
         solana_url: Url,
         sol_ar_url: Url,
         from_keypair: &Keypair,
     ) -> Result<Status, Error> {
+        let mut auto_content_tag = true;
+        let mut status_content_type = mime_guess::mime::OCTET_STREAM.to_string();
+
+        if let Some(content_type) = mime_guess::from_path(file_path.clone()).first() {
+            status_content_type = content_type.to_string();
+            auto_content_tag = false;
+            let content_tag: Tag<Base64> =
+                Tag::from_utf8_strs("Content-Type", &content_type.to_string())?;
+            if let Some(mut tags) = additional_tags {
+                tags.push(content_tag);
+                additional_tags = Some(tags);
+            } else {
+                additional_tags = Some(vec![content_tag]);
+            }
+        }
+
         let transaction = self
             .create_transaction_from_file_path(
                 file_path.clone(),
                 additional_tags,
                 last_tx,
                 price_terms,
+                auto_content_tag,
             )
             .await?;
 
@@ -741,6 +775,7 @@ impl Arweave {
 
         let mut status = Status {
             file_path: Some(file_path),
+            content_type: status_content_type,
             id,
             reward,
             ..Default::default()
@@ -846,16 +881,24 @@ impl Arweave {
         &self,
         data: Vec<u8>,
         mut tags: Vec<Tag<String>>,
+        auto_content_tag: bool,
     ) -> Result<DataItem, Error> {
-        let content_type = if let Some(kind) = infer::get(&data) {
-            kind.mime_type()
-        } else {
-            "application/octet-stream"
-        };
-        tags.extend(vec![
-            Tag::<String>::from_utf8_strs("Content-Type", content_type)?,
-            Tag::<String>::from_utf8_strs("User-Agent", &format!("arloader/{}", VERSION))?,
-        ]);
+        tags.push(Tag::<String>::from_utf8_strs(
+            "User-Agent",
+            &format!("arloader/{}", VERSION),
+        )?);
+
+        // Get content type from [magic numbers](https://developer.mozilla.org/en-US/docs/Web/HTTP/Basics_of_HTTP/MIME_types)
+        // and include additional tags if any.
+        if auto_content_tag {
+            let content_type = if let Some(kind) = infer::get(&data) {
+                kind.mime_type()
+            } else {
+                "application/octet-stream"
+            };
+
+            tags.push(Tag::<String>::from_utf8_strs("Content-Type", content_type)?)
+        }
 
         // let mut anchor = Base64(Vec::with_capacity(32));
         // self.crypto.fill_rand(&mut anchor.0)?;
@@ -883,15 +926,27 @@ impl Arweave {
     pub async fn create_data_item_from_file_path(
         &self,
         file_path: PathBuf,
-        tags: Vec<Tag<String>>,
+        mut tags: Vec<Tag<String>>,
     ) -> Result<(DataItem, Status), Error> {
+        let mut auto_content_tag = true;
+        let mut status_content_type = mime_guess::mime::OCTET_STREAM.to_string();
+
+        if let Some(content_type) = mime_guess::from_path(file_path.clone()).first() {
+            status_content_type = content_type.to_string();
+            auto_content_tag = false;
+            let content_tag: Tag<String> =
+                Tag::from_utf8_strs("Content-Type", &content_type.to_string())?;
+            tags.push(content_tag);
+        }
+
         let data = fs::read(&file_path).await?;
-        let data_item = self.create_data_item(data, tags)?;
+        let data_item = self.create_data_item(data, tags, auto_content_tag)?;
         let data_item = self.sign_data_item(data_item)?;
 
         let status = Status {
             id: data_item.id.clone(),
             file_path: Some(file_path),
+            content_type: status_content_type,
             ..Status::default()
         };
 
@@ -1173,15 +1228,16 @@ impl Arweave {
         let mut consolidated_paths = serde_json::Map::new();
         for (file_path, id_obj) in manifest["paths"].as_object().unwrap() {
             let id = id_obj["id"].as_str().unwrap();
+            let content_type = id_obj["content_type"].as_str().unwrap();
             consolidated_paths.insert(
                 file_path.to_owned(),
-                json!({"id": id, "relative_url": format!(
-                    "https://arweave.net/{}/{}",
-                    transaction_id, file_path
-                ), "id_url": format!(
-                    "https://arweave.net/{}",
-                    id
-                )}),
+                json!({
+                    "id": id,
+                    "files": [
+                        {"uri": format!("https://arweave.net/{}", id), "type": content_type},
+                        {"uri": format!("https://arweave.net/{}/{}", transaction_id, file_path), "type": content_type}
+                    ]
+                }),
             );
         }
         fs::write(
@@ -1193,6 +1249,75 @@ impl Arweave {
         .await?;
         Ok(())
     }
+
+    pub async fn update_metadata_file(
+        &self,
+        file_path: PathBuf,
+        files_array: Vec<Value>,
+        image_link: String,
+    ) -> Result<(), Error> {
+        let data = fs::read_to_string(file_path.clone()).await?;
+        let mut metadata: Value = serde_json::from_str(&data)?;
+        let metadata = metadata.as_object_mut().unwrap();
+        let _ = metadata.insert("image".to_string(), Value::String(image_link));
+
+        let properties = metadata["properties"].as_object_mut().unwrap();
+        let _ = properties.insert("files".to_string(), Value::Array(files_array));
+        // let _ = metadata.insert("properties".to_string(), Value::Object(properties.clone()));
+
+        fs::write(file_path, serde_json::to_string(&json!(metadata))?).await?;
+        Ok(())
+    }
+
+    pub async fn update_metadata<IP>(
+        &self,
+        paths_iter: IP,
+        manifest_path: PathBuf,
+        image_link_file: bool,
+    ) -> Result<(), Error>
+    where
+        IP: Iterator<Item = PathBuf> + Send,
+    {
+        if manifest_path.exists() {
+            let manifest_id = manifest_path
+                .file_stem()
+                .unwrap()
+                .to_str()
+                .unwrap()
+                .split("_")
+                .collect::<Vec<&str>>()
+                .pop()
+                .unwrap();
+            let data = fs::read_to_string(manifest_path.clone()).await?;
+            let mut manifest: Value = serde_json::from_str(&data)?;
+            let manifest = manifest.as_object_mut().unwrap();
+
+            try_join_all(paths_iter.map(|p| {
+                let path_object = manifest.get(p.to_str().unwrap()).unwrap();
+                let image_link = if image_link_file {
+                    format!(
+                        "https://arweave.net/{}/{}",
+                        manifest_id,
+                        p.to_str().unwrap()
+                    )
+                } else {
+                    format!(
+                        "https://arweave.net/{}",
+                        path_object["id"].as_str().unwrap()
+                    )
+                };
+                self.update_metadata_file(
+                    p.with_extension("json"),
+                    path_object["files"].as_array().unwrap().clone(),
+                    image_link,
+                )
+            }))
+            .await?;
+            Ok(())
+        } else {
+            Err(Error::ManifestNotFound)
+        }
+    }
 }
 
 #[cfg(test)]
@@ -1223,7 +1348,13 @@ mod tests {
         let last_tx = Base64::from_str("LCwsLCwsLA")?;
         let other_tags = vec![Tag::<Base64>::from_utf8_strs("key2", "value2")?];
         let transaction = arweave
-            .create_transaction_from_file_path(file_path, Some(other_tags), Some(last_tx), (0, 0))
+            .create_transaction_from_file_path(
+                file_path,
+                Some(other_tags),
+                Some(last_tx),
+                (0, 0),
+                true,
+            )
             .await?;
 
         let error = arweave.post_transaction(&transaction).await.unwrap_err();
@@ -1251,6 +1382,7 @@ mod tests {
                 Some(other_tags),
                 Some(last_tx),
                 (0, 0),
+                true,
             )
             .await?;
 
@@ -1372,4 +1504,24 @@ mod tests {
         assert_eq!((10, 18265), (number_of_files, data_size));
         Ok(())
     }
+
+    #[test]
+    fn test_mime_types() -> Result<(), Error> {
+        let file_paths = vec![
+            "some.png",
+            "some.jpg",
+            "some.json",
+            "some.txt",
+            "some.css",
+            "some.js",
+        ];
+
+        let paths_iter = file_paths.iter().map(|p| PathBuf::from(p));
+
+        for p in paths_iter {
+            println!("{}", mime_guess::from_path(p).first().unwrap());
+        }
+
+        Ok(())
+    }
 }
